mcc.f<-function(x, var1, var2)
{
	s<-var(x)
	s0<-sqrt(s[var1,var1])
	s0q<-s[var1,var2]
	Sq<-s[var2,var2]
	
	sqrt(s0q%*%solve(Sq)%*%s0q)/s0
}


pcc.f<-function(x,z1,z2,x2)
{
	x<-as.matrix(x)
	s<-var(x)
	s22<-s[x2,x2]

	y1<-x[,z1]-x[,x2]%*%solve(s22)%*%s[z1,x2]
	y2<-x[,z2]-x[,x2]%*%solve(s22)%*%s[z2,x2]
	cor(y1,y2)
}


andrews.f<-function(x,v)
{
 
	n<-nrow(x) 
	p<-ncol(x)
	nv<-length(v)

	x<-as.matrix(x)
	m<-cov.wt(x)$center
	M<-matrix(m*rep(1,n*p),n,p,byrow=T)
	d<-diag(1/sqrt(diag(cov.wt(x)$cov)))

	x<-(x-M)%*%d

	l<-100
	tt<-seq(from=-pi,to=pi,length=l);
	andr.v<-c(rep(1/sqrt(2),l))

	for(i in 1:((p-1)/2)) 
		andr.v<-c(andr.v,sin(i*tt),cos(i*tt))
  
	if( p%%2 == 0 ) 
		andr.v<-andr.v[-( ((p-1)*l+1): p*l )]

	andr.m<-matrix(andr.v,nrow=l,ncol=p)

	y <- andr.m%*%t(x)
	z <- matrix(0,nv*l,2)
	lab<-vector("numeric",nv*l)

 
	plot(rep(tt,n),as.vector(y),type="n",xlab="t",ylab="Andrews plot")
	for(i in 1:nv) {
		lines(tt,y[,v[i]])
		for(j in 1:l ) {
			z[(i-1)*l+j,1]<-tt[j]
			z[(i-1)*l+j,2]<-y[j,v[i]]
			lab[(i-1)*l+j]<-v[i]
		}
	}
	abline(h=0)
	identify(z,labels=lab)
}

skew.f<-function(x)
{
	x<-as.matrix(x)
	n<-nrow(x)
	m<-cov.wt(x)$center
	S<-cov.wt(x,method="ML")$cov
	SI<-solve(S)

	skew<-0		
	for(r in 1:n) {
		for(s in 1:n) {
			skew<-skew+(t(x[r,]-m)%*%SI%*%(x[s,]-m))^3
		}
        }
	skew/(n^2)
}


kurtosis.f<-function(x)
{
	x<-as.matrix(x)
	n<-nrow(x)
	m<-cov.wt(x)$center
	S<-cov.wt(x,method="ML")$cov
	SI<-solve(S)


	kurtosis<-0
	for(r in 1:n) {
		kurtosis<-kurtosis+( t(x[r,]-m) %*% SI %*% (x[r,]-m) )^2
	}
	kurtosis/n
}



normality.f<-function(x,alpha=0.05)
{
	n<-nrow(x)
	p<-ncol(x)
	m<-(p*(p+1)*(p+2))/6
	b1<-skew.f(x)
	p.val<-1-pchisq((n*b1)/6,m)
	cat("Skewness is ",b1,"and P-value is ",p.val,".\n")

	if(p.val < alpha)	
		cat("Reject the normality.\n")
	else 
		cat("Don't reject the normality.\n")
}



chiplot.f<-function(x)
{
	n<-nrow(x) 
	p<-ncol(x)

	mah<-sort(mahalanobis(x,mean(x),var(x)))
	arr<-seq(from=1/(2*n),length=n,by=1/n); 
	chi.per<-qchisq(arr,p)
	
	plot(mah,chi.per,xlab="Ordered Mahalanobis distance",ylab="Chi-sqare percentile");
}



betaplot.f<-function(x)
{
	n<-nrow(x)
	p<-ncol(x)

	alpha<-(p-2)/(2*p)
	beta<-(n-p-3)/(2*(n-p-1))
	
	mah<-sort((n/(n-1)^2)*mahalanobis(x,mean(x),var(x)))

	start<-(1-alpha)/(n-alpha-beta+1)
	step<-1/(n-alpha-beta+1)	
	arr<-seq(from=start,length=n,by=step)
	beta.per<-qbeta(arr,alpha,beta)

	plot(mah,beta.per,xlab="Ordered b_r",ylab="Beta percentile") 
}



mult.f<-function(lambda, x) 
{
	n<-nrow(x)
	p<-ncol(x)
	H<-rep(1,n)%*%t(rep(1,n))/n

	Y<-matrix(0,n,p)
	for(r in 1:n) { 
		for(i in 1:p) {
			if( lambda[i] == 0 ) 
				Y[r,i]<-log(x[r,i])
			else 
				Y[r,i]<-(x[r,i]^lambda[i] - 1)/lambda[i]
		}
	}

	Z<-matrix(0,n,p)
	for(i in 1:p) {
		a<-(prod(x[,i]))^((lambda[i]-1)/n)
		Z[,i]<-Y[,i]/a
	}

	log(prod(eigen(t(Z)%*%(diag(n)-H)%*%Z)$values))
}



univ.f<-function(lambda, x) 
{
	n<-length(x)
	H<-rep(1,n) %*% t(rep(1,n))/n

	Y<-rep(0,n)
	for(r in 1:n) {
		if( lambda == 0 ) 
			Y[r]<-log(x[r])
		else 
			Y[r]<-(x[r]^lambda-1)/lambda
	}

	Y<-Y/(prod(x)^((lambda-1)/n))
	t(Y)%*%(diag(n)-H)%*%Y
}


boxcox.f<-function(x)  
{ 
	x<-as.matrix(x)
	p<-ncol(x)
	sp<-rep(1,p)

	for(i in 1:p) {
		sp[i]<-nlminb(objective=univ.f,start=sp[i],x=x[,i])$par
	}

	lambda=nlminb(objective=mult.f,start=sp,x=x)$par
	cat("Box-Cox parameters : ",lambda,"\n");

}	


hotelling1.f<-function(x,mu0,alpha=0.05) 
{
	n<-nrow(x)
	p<-ncol(x)
	m<-mean(x)
	s<-var(x)
	si<-solve(s)

	t2<-(n-p)/(p*(n-1))*n*t(m-mu0)%*%si%*%(m-mu0)
	p.val<-1-pf(t2,p,n-p)	
	cat("The test statistic is ",t2,"and P-value is ",p.val,".\n") 

	if(p.val < alpha) 
		cat("Reject the null hypothesis.\n")
	else
		cat("Don't reject the null hypothesis.\n")
}


linhypo.f<-function(x,A,b,alpha=0.05)
{
	n<-nrow(x)
	q<-qr(A)$rank
	m<-mean(x)
	iv<-solve(A%*%var(x)%*%t(A))

	t2<-(n-q)/(q*(n-1))*n*t(A%*%m-b)%*%iv%*%(A%*%m-b)
	p.val<-1-pf(t2,q,n-q)	
	cat("The test statistic is ",t2,"and P-value is ",p.val,".\n") 

	if(p.val < alpha) 
		cat("Reject the null hypothesis.\n")
	else
		cat("Don't reject the null hypothesis.\n")
}


indep.f<-function(x,g1,g2,alpha=0.05) 
{
	p1<-length(g1)
	p2<-length(g2)
	n<-nrow(x)

	s<-cov.wt(x,method="ML")$cov
	s11.i<-solve(s[g1,g1])
	s22.i<-solve(s[g2,g2])
	s12<-s[g1,g2]
	s21<-s[g2,g1]

	a<-diag(p2)-s22.i%*%s21%*%s11.i%*%s12
	lrt<--n*sum(log(eigen(a)$val))

	p.val<-1-pchisq(lrt,p1*p2)
	cat("The LRT is ",lrt,"and P-value is ",p.val,".\n")
	if(p.val < alpha)
		cat("Reject the null hypothesis.\n")
	else
		cat("Don't reject the null hypothesis.\n")
}



equicorr.f<-function(x,alpha=0.05)
{
	n<-nrow(x)
	p<-ncol(x)

	if(p <=2 ) 
		stop("Dimension must be larger than 2\n")
	
	sn<-cov.wt(x,method="ML")$cov
	sigma2<-sum(eigen(sn)$val)/p
	rho<-(sum(sn)-sum(diag(sn)))/(p*(p-1)*sigma2)
	Sigma<-sigma2*((1-rho)*diag(p)+rho*matrix(rep(1,p*p),p,p))
	Sigma.i<-solve(Sigma)
	a<-(Sigma.i%*%sn)
	eig.val<-eigen(a)$val
	lrt<--n*sum(log(eig.val))+n*sum(eig.val)-n*p
	df<-p*(p+1)/2-2
	p.val<-1-pchisq(lrt,df)

	cat("The LRT is ",lrt,"and P-value is ",p.val,".\n")
	if(p.val < alpha)
		cat("Reject the null hypothesis.\n")
	else 
		cat("Don't reject the null hypothesis.\n")
}


hotelling2.f<-function(x1,x2,alpha=0.05)
{
	n1<-nrow(x1) 
	n2<-nrow(x2)
	n<-n1+n2
	p<-ncol(x1)
	
	m1<-mean(x1)
	m2<-mean(x2)
	s1<-cov.wt(x1,method="ML")$cov
	s2<-cov.wt(x2,method="ML")$cov
	sp<-(n1*s1+n2*s2)/n
	
	t2<-(n1*n2*(n-p-1))/(n^2*p)*t(m1-m2)%*%solve(sp)%*%(m1-m2)
	p.val<-1-pf(t2,p,n-p-1)	
	cat("The test statistic is ",t2,"and P-value is ",p.val,".\n") 

	if(p.val < alpha) 
		cat("Reject the null hypothesis.\n")
	else
		cat("Don't reject the null hypothesis.\n")
}


bfp.f<-function(x1,x2,tol=0.001)
{
	n1<-nrow(x1)
	n2<-nrow(x2)
	p<-ncol(x1)
	m1<-mean(x1) 
	m2<-mean(x2)
	S1<-cov.wt(x1,method="ML")$cov; 
	S2<-cov.wt(x2,method="ML")$cov

	m.prev<-rep(0,p)
	m.curr<-solve(n1*solve(S1)+n2*solve(S2))%*%(n1*solve(S1)%*%m1+n2*solve(S2)%*%m2)
	m.curr<-as.vector(m.curr)
	
	while( dist(rbind(m.prev,m.curr)) > tol ) {
		m.prev<-m.curr
		S1.curr<-S1 + (m1 - m.curr)%*%t(m1 - m.curr)
		S2.curr<-S2 + (m2 - m.curr)%*%t(m2 - m.curr)
		m.curr<-solve(n1*solve(S1.curr)+n2*solve(S2.curr))%*%(n1*solve(S1.curr)%*%m1+n2*solve(S2.curr)%*%m2)
		m.curr<-as.vector(m.curr)
	}

	list(m=m.curr,S1=S1.curr,S2=S2.curr)	
}



covh3.f<-function(x1,x2,tol=0.001)
{
	n1<-nrow(x1)
	n2<-nrow(x2)
	p<-ncol(x1)
	S1<-cov.wt(x1,method="ML")$cov; 
	S2<-cov.wt(x2,method="ML")$cov

	r1<-n1/(n1+n2)
	r2<-n2/(n1+n2)
	D.prev<-matrix(0,p,p)
	D.curr<-diag(p)

	while( dist( rbind(diag(D.prev),diag(D.curr)) ) > tol ) {
		Di<-solve(D.curr)
		S<-r1*S1 + r2*Di%*%S2%*%Di
		D.prev<-D.curr
		D.curr<-diag(diag(solve(S)%*%Di%*%S2))
	}

	colnames(D.curr)<-colnames(S); 
	rownames(D.curr)<-rownames(S)
	list(D=D.curr,S=S)	
}


covh2.f<-function(x1,x2,tol=0.001) 
{
	n1<-nrow(x1)
	n2<-nrow(x2)
	p<-ncol(x1)
	S1<-cov.wt(x1,method="ML")$cov
	S2<-cov.wt(x2,method="ML")$cov

	r1<-n1/(n1+n2)
	r2<-n2/(n1+n2)

	c.prev<-0
	c.curr<-sum(eigen(solve(S1)%*%S2)$val)/p

	while( abs(c.curr-c.prev) > tol ) {
		S<-r1*S1 + r2*S2/c.curr
		c.prev<-c.curr
		c.curr<-sum(eigen(solve(S)%*%S2)$val)/p
	}
	list(c=c.curr,S=S)	
}


locinf.f<-function(x)
{
	x<-as.matrix(x)
	n<-nrow(x)
	m<-cov.wt(x)$center
	S<-cov.wt(x,method="ML")$cov
	Si<-solve(S)

	f<-matrix(0,n,n)
	for(r in 1:n){
		for(s in 1:n){
			d<-t(x[r,]-m)%*%Si%*%(x[s,]-m)
			f[r,s]<-2*(d+d^2/2)/n
		}
	}

	a<-eigen(f,symmetric=T)
	b<-order(abs(a$val),decreasing=T)
	max<-b[1]; sec<-b[2]	
	val<-c(a$values[max],a$values[sec])
	e1<-a$vectors[,max]
	e2<-a$vectors[,sec]

	plot(e1,e2,type="n",xlab="First direction",ylab="Second direction")
	text(e1,e2)
	abline(h=0,v=0)	
	list(val=val, lmax=e1, lsec=e2)
}


lambstar.f<-function(lamb, p, q, r)
{
   g<-(p*r-2)/2
   t<-sqrt((p^2*r^2-4)/(p^2+r^2-5))
   f<-q-(p-r+1)/2
   n1<-p*r
   n2<-f*t-g
   lambstar<-(1-lamb^(1/t))/lamb^(1/t)*n2/n1
   list(lambstar=lambstar,n1=n1,n2=n2)
}


mmrlh.f<-function(X, Y, G, C)
{
  n<-nrow(X)
  X1<-cbind(rep(1,n), X)
  lmrts<-lm(Y~X)
  ps1<-lmrts$coef
  E1<-lmrts$resid
  W1<-t(E1) %*% E1
  ps0<-ps1 + solve(t(X1) %*% X1) %*% t(G)%*% solve(G %*% solve(t(X1) %*% X1) %*% t(G))%*% (C - G %*% ps1)
  E0<-Y - X1 %*% ps0
  S0<-t(E0) %*% E0/n
  W0<-W1 + t(C - G %*% ps1) %*% solve(G %*% solve(t(X1) %*% X1)%*% t(G)) %*% (C - G %*% ps1)
  D<-W0 - W1
  V<-eigen(W0)$vec
  L<-solve(diag(sqrt(eigen(W0)$val)))
  W0.sqi<-V %*% L %*% t(V)
  WDW<-W0.sqi %*% D %*% W0.sqi
  gamma<-eigen(WDW)$val
  Wilks<-prod(1 - gamma)

  list(psi0 = ps0, S0 = S0,gamma = gamma, Wilks = Wilks)
}


mmrmult.f<-function(lambda,x,y)
{
    n<-nrow(x)
    p<-ncol(y)
    x<-cbind(rep(1,n),x)

    H<-x%*%solve(t(x)%*%x)%*%t(x)

    Y<-matrix(0,n,p)
    for(r in 1:n) {
        for(i in 1:p) {
            if( lambda[i] == 0 )
                Y[r,i]<-log(y[r,i])
            else
                Y[r,i]<-(y[r,i]^lambda[i] - 1)/lambda[i]
        }
    }

    Z<-matrix(0,n,p)
    for(i in 1:p) {
        a<-(prod(y[,i]))^((lambda[i]-1)/n)
        Z[,i]<-Y[,i]/a
    }

    log(prod(eigen(t(Z)%*%(diag(n)-H)%*%Z)$values))
}


mmruniv.f<-function(lambda,x,y)
{
    n<-length(y)
    x<-cbind(rep(1,n),x)
    H<-x%*%solve(t(x)%*%x)%*%t(x)

    Y<-rep(0,n)
    for(r in 1:n) {
        if( lambda == 0 ) Y[r]<-log(y[r])
        else Y[r]<-(y[r]^lambda-1)/lambda
    }

    Y<-Y/(prod(x)^((lambda-1)/n))
    t(Y)%*%(diag(n)-H)%*%Y
}



mmrboxcox.f<-function(x,y) 
{
    x<-as.matrix(x)
    y<-as.matrix(y)
    p<-ncol(y)
    sp<-rep(1,p)
    for(i in 1:p)
        sp[i]<-nlminb(objective=mmruniv.f,start=sp[i],x=x,y=y[,i])$par

    cat("Box-Cox parameters : ",
      nlminb(objective=mmrmult.f,start=sp,x=x,y=y)$par,"\n")
}


ldf.f<-function(x1,x2)
{
	x1<-as.matrix(x1)
	x2<-as.matrix(x2)
	n1<-nrow(x1)
	n2<-nrow(x2)
	n<-n1+n2
	r1<-n1/n
	r2<-1-r1
	A1<-cov.wt(x1,method="ML")
	A2<-cov.wt(x2,method="ML")
	m1<-A1$center
	m2<-A2$center
	S1<-A1$cov
	S2<-A2$cov
	sp<-r1*S1+r2*S2
	sp.i<-solve(sp)
	c1<-t(m1-m2)%*%sp.i
	c2<--0.5*t(m1-m2)%*%sp.i%*%(m1+m2)-log(r2/r1)
	mp<-t(m1-m2)%*%sp.i%*%(m1-m2)
	mp<-pnorm(-sqrt(mp)/2)

	out1<-c1%*%t(x1)+rep(c2,n1);out2<-c1%*%t(x2)+rep(c2,n2)
	class<-c(out1,out2)
	class<-as.numeric(class<0)+1 #>0 : 1 and <0 : 2
			

	list(c1=c1,c2=c2,out1=out1,out2=out2,mp=mp,class=class)
}


qdf.f<-function(x1, x2) 
{
	x1<-as.matrix(x1)
	x2<-as.matrix(x2)
	n1<-nrow(x1)
	n2<-nrow(x2)
	n<-n1+n2
	r1<-n1/(n1+n2)
	r2<-1-r1

	A1<-cov.wt(x1,method="ML")
	A2<-cov.wt(x2,method="ML")
	m1<-A1$center
	m2<-A2$center
	s1<-A1$cov
	s2<-A2$cov
	s1.i<-solve(s1)
	s2.i<-solve(s2) 


	c1<--0.5*(s1.i-s2.i)
	c2<-s1.i%*%m1-s2.i%*%m2
	c3<-0.5*( log(det(s2)/det(s1))-t(m1)%*%s1.i%*%m1+t(m2)%*%s2.i%*%m2 ) - log(r2/r1)


	out1<-t(diag(x1%*%c1%*%t(x1))+x1%*%c2+rep(c3,n1))
	out2<-t(diag(x2%*%c1%*%t(x2))+x2%*%c2+rep(c3,n2))

	class<-c(out1,out2)
	class<-as.numeric(class<0) + 1 # >0:1 and <0 :2

	list(c1=c1, c2=c2, c3=c3, out1=out1, out2=out2, class=class)
}



mdf.f<-function(x,gr)
{
	x<-as.matrix(x)
	n<-nrow(x)
	p<-ncol(x)
	
	n.gr<-length(levels(gr))	
	r<-rep(0,n.gr) # r_i
	s.p<-matrix(0,p,p)	

	a<-by(x,gr,cov.wt,method="ML")
	for(i in 1:n.gr) {
		r[i]<-a[[i]]$n.obs/n
		s.p<-s.p+r[i]*a[[i]]$cov
	}
	s.p.i<-solve(s.p)


	lf<-rep(0,n.gr) # likelihood ftn
	out<-rep(0,n) # allocation index
	for(i in 1:n){
		for(j in 1:n.gr) {
			m<-a[[j]]$center
			lf[j]<-log(r[j])+t(m)%*%s.p.i%*%(x[i,]-0.5*m)
		}
		out[i]<-order(lf,decreasing=T)[1]	
	}

	list(class=out)
}



varsel.f<-function(x1, x2, var.vec, alpha=0.05)
{
	p<-ncol(x1)
	k<-length(var.vec)

	n1<-nrow(x1)
	n2<-nrow(x2)
	n<-n1+n2
	m<-(n*(n-2))/(n1*n2)

	#D^2_p
	m1<-mean(x1)
	m2<-mean(x2)
	sp<-(n1*var(x1)+n2*var(x2))/(n-2)
	sp.i<-solve(sp)
	D2.p<-t(m1-m2)%*%sp.i%*%(m1-m2)

	#D^2_k
	x3<-x1[,var.vec]
	x4<-x2[,var.vec]
	m3<-mean(x3)
	m4<-mean(x4)
	sp.k<-(n1*var(x3)+n2*var(x4))/(n-2)
	sp.k.i<-solve(sp.k)
	D2.k<-t(m3-m4)%*%sp.k.i%*%(m3-m4)

	#F
	F.stat<-((n-p-1)*(D2.p-D2.k))/((p-k)*(m+D2.k))
	p.val<-1-pf(F.stat,p-k,n-p-1)
	cat("F-Statistic is ",F.stat," and P-value is ",p.val,"\n")
	if(p.val < alpha)
		cat("Reject the null hypothesis.\n")
	else
		cat("Don't reject the null hypothesis.\n")
}


ccaload.f<-function(x,y)
{
	x<-as.matrix(x)
	y<-as.matrix(y)
	cca<-cancor(x,y)
	
	u<-x%*%cca$xcoef
	v<-y%*%cca$ycoef
	cor(cbind(x,y), cbind(u,v))
}
















